******************************************************************************
*                                                                            *
*   Secuencia del Programa                                                   *
*       Reset:  Llama a get_char para recibir un byte de dato                *
*               Enciende el LED frame error si ocurre un error de frame      *
*               Enciende el LED de Noise si hay ruido en la lines            *
*               Llama a put_char para transmitir el dato recibidota          *
*               Vuelve otra vez al llamado a get_char (loop infinito)        *
*                                                                            *
******************************************************************************
*   PORTB :  Bit1 - Recibe Dato                                              *
*            Bit2 - Transmite Dato                                           *
*            Bit6 - Led de frame error                                       *
*            Bit7 - Led de noise error                                       *
******************************************************************************

$Include 'jl3regs.inc'

RamStart   EQU    $0080
RomStart   EQU    $F600
VecSpace   EQU    $FFDE

RXD        EQU    1                    ; pin # for receive data pin
TXD        EQU    2                    ; pin # for transmit data pin

NOISE      EQU    6                    ; pin # for noise LED
FRAME      EQU    7                    ; pin # for frame LED


*** Program Constant Equates:          ; Baud rate select table:
BAUD_SEL   EQU    $12                  ; BAUD_SEL   4.9MHz
                                       ;    $04     19.2k
                                       ;    $08      9600
                                       ;    $10      4800
                                       ;    $20      2400
                                       ;    $40      1200
                                       ;    $80       600

* ------------------------------------------------------------
*  Salidas de PORTB

CLK        EQU    5
DTA        EQU    6
ANOD       EQU    7

* ------------------------------------------------------------
* RAM Variables
* ------------------------------------------------------------

           ORG    RamStart

CHAR       RMB    1                    ; data register for sci
COUNT      RMB    1                    ; temp storage variable

********************************************************************************
* Start - Programa de ejemplo que continuamente transmite el caracter recibido *
*                                                                              *
* input cond.     - Start                                                      *
* output cond.    - none (loop infinito)                                       *
* stack used      - 4 bytes                                                    *
* variables used  - none                                                       *
* ROM used        - 28 bytes                                                   *
********************************************************************************

           ORG    RomStart

start      RSP
           MOV    #%11000100,DDRB
           MOV    #%11000100,PORTB     ; noise + frame flags = off

MLOOP      JSR    GET_CHAR

           BCC    NOFERR
           BCLR   FRAME,PORTB
           BRA    CONTINUE             ; don't check for noise --

NOFERR     BSET   FRAME,PORTB

           BHCS   NOISERR
           BSET   NOISE,PORTB
           BRA    CONTINUE

NOISERR    BCLR   NOISE,PORTB          ; turn on noise LED

CONTINUE:
           JSR    PUT_CHAR
           BRA    MLOOP

******************************************************************************
* get_char - receive one byte of data from RXD pin; called by main           *
*                                                                            *
* input cond.     - RXD pin defined as an input pin                          *
* output cond.    - char contains received data; X,ACC undefined;            *
*                   half carry = 1 (frame occured) or 0 (no frame error);    *
*                   carry = 1 (noise and/or frame error occured) or 0        *
*                   (no noise).                                              *
* stack used      - 2 bytes                                                  *
* variables used  - char: storage for received data (1 byte)                 *
*                   count: temporary storage (1 byte)                        *
* ROM used        - 63 bytes                                                 *
******************************************************************************

GET_CHAR   LDA    #8
           STA    COUNT
           CLRX

GET_START  BRCLR  RXD,PORTB,*
           BRSET  RXD,PORTB,*

           LDA    #BAUD_SEL-3
           BSR    DELAY
           BSR    GET_BIT
           LSRA                        ; noise bit -> carry;

           BNE    GET_START
           TSTA
           TSTA

           LDA    #2*(BAUD_SEL-2)
           BSR    DELAY

get_data   BSR    GET_BIT
           RORA                        ; noise bit -> carry
           RORX                        ; carry -> noise data reg
           RORA                        ; filtered data bit -> carry
           ROR    CHAR                 ; carry -> char
           LDA    #2*(BAUD_SEL-3)
           BSR    DELAY
           TSTA
           DEC    COUNT
           BNE    GET_DATA

GET_STOP   BSR    GET_BIT
           LSRA                        ; noise bit -> carry
           STA    COUNT                ; store stop bit in count
           BCC    YES_NOISE

           TXA
           EOR    CHAR
           BEQ    NO_NOISE

YES_NOISE  LDA    #$08                 ; set noise bit (half carry)
           ADD    #$08                 ; by adding $8 to $8

NO_NOISE   LDA    COUNT                ; retrieve stop data bit,
           COMA
           LSRA
           RTS

******************************************************************************


******************************************************************************
* get_bit - receive one bit of filtered data and noise info; called by       *
*            get_char                                                        *
*                                                                            *
* input cond.     - RXD pin defined as an input pin                          *
* output cond.    - ACC = 000000dn, where d = filtered data, n = noise info  *
* stack used      - 0 bytes                                                  *
* variables used  - none                                                     *
* ROM used        - 17 bytes                                                 *
******************************************************************************

GET_BIT    CLRA
           BRSET  RXD,PORTB,SAMP_1
SAMP_1     ADC    #0
           BRSET  RXD,PORTB,SAMP_2
SAMP_2     ADC    #0
           BRSET  RXD,PORTB,SAMP_3
SAMP_3     ADC    #0
           RTS

******************************************************************************
* put_char - transmit data byte in char out onto TXD pin; called by main     *
*                                                                            *
* input cond.     - TXD pin defined as an output pin and TXD = 1;            *
*                   char contains byte to be tranmitted.                     *
* output cond.    - X,ACC,char =  undefined;                                 *
* stack used      - 2 bytes                                                  *
* variables used  - char: storage for transmitted data (1 byte)              *
* ROM used        - 31 bytes                                                 *
******************************************************************************

PUT_CHAR:
           LDX    #9
           CLC

PUT_DATA   BCC    SEND_0
           BSET   TXD,PORTB
           BRA    JMP_BIT

SEND_0     BCLR   TXD,PORTB
           BRA    JMP_BIT

JMP_BIT    LDA    #2*(BAUD_SEL-1)-1
           BSR    DELAY
           TSTA
           ROR    CHAR
           DECX
           BNE    PUT_DATA

           NOP
           BSET   TXD,PORTB
           LDA    #2*(BAUD_SEL-1)
*-----
DELAY      NOP
           NOP
           TSTA
           DECA
           BNE    DELAY
           RTS

* ---------------------------------------------------

           ORG    VECSPACE

           FDB    start                 ; ADC Conversion Complete Vector
           FDB    start                 ; Keyboard Vector
           FDB    0,0,0,0,0,0,0,0       ; Not used
           FDB    start                 ; TIM1 Overflow Vector
           FDB    start                 ; TIM1 Channel 1 Vector
           FDB    start                 ; TIM1 Channel 0 Vector
           FDB    start                 ; (No Vector Assigned $FFF8-$FFF9)
           FDB    start                 ; ~IRQ1
           FDB    start                 ; SWI Vector
           FDB    start                 ; Reset Vector
